@@ -34,7 +34,7 @@ def checkInfile(filename):
 # action: True  is for PUT/POST HTTP method
 #         False is for DELETE HTTP method
 def setJSON(domain, action, filename=False, record=False):
-    import tonicdnscli.converter as converter
+    import converter
     o = converter.JSONConvert(domain)
 
     # for 'bulk_create/bulk_delete'
@@ -138,7 +138,8 @@ def create(args):
     action = True
 
     # for create sub-command
-    if args.__dict__.get('domain'):
+    if (args.__dict__.get('domain') and args.__dict__.get('name')
+        and args.__dict__.get('rtype') and args.__dict__.get('content')):
 
         from converter import JSONConvert
         domain = args.domain
@@ -151,8 +152,10 @@ def create(args):
 
     # for bulk_create sub-command
     else:
-
-        domain = checkInfile(args.infile)
+        if args.__dict__.get('domain'):
+            domain = args.domain
+        else:
+            domain = checkInfile(args.infile)
         json = setJSON(domain, action, filename=args.infile)
 
     password = getPassword(args)
@@ -168,7 +171,8 @@ def delete(args):
     action = False
 
     # for delete sub-command
-    if args.__dict__.get('domain'):
+    if (args.__dict__.get('domain') and args.__dict__.get('name')
+        and args.__dict__.get('rtype') and args.__dict__.get('content')):
 
         from converter import JSONConvert
         domain = args.domain
@@ -181,8 +185,10 @@ def delete(args):
 
     # for bulk_delete sub-command
     else:
-
-        domain = checkInfile(args.infile)
+        if args.__dict__.get('domain'):
+            domain = args.domain
+        else:
+            domain = checkInfile(args.infile)
         json = setJSON(domain, action, filename=args.infile)
 
     password = getPassword(args)
@@ -206,37 +212,6 @@ def template_get(args):
         processing.getAllTemplates(args.server, t)
 
 
-# Create or update template
-def template_create_or_update(args):
-    import processing
-    import converter
-
-    domain = args.domain
-
-    # When specify '--template' option
-    if args.__dict__.get('template'):
-        identifier = args.template
-
-    else:
-        identifier = domain.replace('.', '_')
-
-    o = converter.JSONConvert(domain)
-    dnsaddr = args.dnsaddr
-    desc = args.desc if args.__dict__.get('desc') else ''
-
-    password = getPassword(args)
-    t = token(args.username, password, args.server)
-    o.generateTemplate(domain, dnsaddr, desc=desc)
-
-    # When update template
-    if args.__dict__.get('template'):
-        processing.updateTemplate(args.server, t, identifier, o.record)
-
-    # When create template
-    else:
-        processing.createTemplate(args.server, t, identifier, o.record)
-
-
 # Delete template
 def template_delete(args):
     import processing
@@ -256,11 +231,74 @@ def updateSOASerial(args):
 
     password = getPassword(args)
     t = token(args.username, password, args.server)
+    soa_content = dict(domain=args.domain)
+    if args.__dict__.get('mname'):
+        soa_content['mname'] = args.mname
+    if args.__dict__.get('rname'):
+        soa_content['rname'] = args.rname
+    if args.__dict__.get('refresh'):
+        soa_content['refresh'] = args.refresh
+    if args.__dict__.get('retry'):
+        soa_content['retry'] = args.retry
+    if args.__dict__.get('expire'):
+        soa_content['expire'] = args.expire
+    if args.__dict__.get('minimum'):
+        soa_content['minimum'] = args.minimum
+    processing.updateSerial(args.server, t, soa_content)
+
+
+# Create zone
+def createZone(args):
+    import processing
+    from converter import JSONConvert
+
+    action = True
+
+    password = getPassword(args)
+    t = token(args.username, password, args.server)
+
     domain = args.domain
-    processing.updateSerial(args.server, t, domain)
+    template = args.domain.replace('.', '_')
+
+    master = None
+    dnsaddr = args.dnsaddr
+    if args.__dict__.get('S'):
+        dtype = 'SLAVE'
+        master = dnsaddr
+    elif args.__dict__.get('N'):
+        dtype = 'NATIVE'
+    else:
+        dtype = 'MASTER'
+
+    # generate template data
+    o = JSONConvert(domain)
+    o.generateTemplate(domain, dnsaddr, desc='')
+
+    # create template
+    processing.createTemplate(args.server, t, template, o.record)
+
+    # create zone
+    processing.createZoneRecords(
+        args.server, t, domain, template, dtype, master)
+
+    # delete template
+    processing.deleteTemplate(args.server, t, template)
 
 
-def setoption(obj, keyword, prefix=False, required=False):
+# Delete zone
+def zone_delete(args):
+    import processing
+    import converter
+
+    if args.__dict__.get('domain'):
+        domain = args.domain
+
+    password = getPassword(args)
+    t = token(args.username, password, args.server)
+    processing.deleteZone(args.server, t, domain)
+
+
+def setoption(obj, keyword, required=False):
     if keyword == 'server':
         obj.add_argument(
             '-s', dest='server', required=True,
@@ -297,8 +335,6 @@ def setoption(obj, keyword, prefix=False, required=False):
 
     if keyword == 'template':
         msg = 'specify template identifier'
-        if prefix:
-            msg = prefix + msg
         if required:
             obj.add_argument('--template', action='store',
                              required=True, help=msg)
@@ -311,19 +347,21 @@ def setoption(obj, keyword, prefix=False, required=False):
                          help='partial match search')
 
 
-def conn_options(obj, server=False, username=False, password=False):
-    if server and username and password:
-        obj.set_defaults(server=server, username=username,
-                         password=password)
+def conn_options(obj, conn):
+    if conn.get('server') and conn.get('username') and conn.get('password'):
+        obj.set_defaults(server=conn.get('server'),
+                         username=conn.get('username'),
+                         password=conn.get('password'))
 
-    elif server and username:
-        obj.set_defaults(server=server, username=username)
+    elif conn.get('server') and conn.get('username'):
+        obj.set_defaults(server=conn.get('server'),
+                         username=conn.get('username'))
 
-    if not server:
+    if not conn.get('server'):
         setoption(obj, 'server')
-    if not username:
+    if not conn.get('username'):
         setoption(obj, 'username')
-    if not password:
+    if not conn.get('password'):
         setoption(obj, 'password')
 
 
@@ -343,91 +381,168 @@ def parse_options():
         if os.path.isfile(CONFIGFILE):
             server, username, password = checkConfig(CONFIGFILE)
 
+    conn = dict(server=server, username=username, password=password)
+
     subprs = prs.add_subparsers(help='commands')
 
     # Convert and print JSON
-    prs_show = subprs.add_parser('show',
+    prsShow(subprs)
+
+    # Retrieve records
+    prsGet(subprs, conn)
+
+    # Create record
+    prsCreate(subprs, conn)
+
+    # Create bulk_records
+    prsBulkCreate(subprs, conn)
+
+    # Delete record
+    prsDelete(subprs, conn)
+
+    # Delete bulk_records
+    prsBulkDelete(subprs, conn)
+
+    # Update SOA serial
+    prsSOAUpdate(subprs, conn)
+
+    # Create zone
+    prsZoneCreate(subprs, conn)
+
+    # Delete zone
+    prsZoneDelete(subprs, conn)
+
+    # Retrieve template
+    prsTmplGet(subprs, conn)
+
+    # Delete template
+    prsTmplDelete(subprs, conn)
+
+    args = prs.parse_args()
+    return args
+
+
+# Convert and print JSON
+def prsShow(obj):
+    prs_show = obj.add_parser('show',
                                     help='show converted JSON')
     setoption(prs_show, 'infile')
     prs_show.set_defaults(func=show)
 
-    # Retrieve records
-    prs_get = subprs.add_parser(
-        'get', help='retrieve all zones without a specific zone,\
-or records with a specific zone')
+
+# Retrieve records
+def prsGet(obj, conn):
+    prs_get = obj.add_parser(
+        'get', help='retrieve all zones or records with a specific zone')
     prs_get.add_argument('--domain', action='store',
                          help='specify domain FQDN')
-    conn_options(prs_get, server, username, password)
+    conn_options(prs_get, conn)
     setoption(prs_get, 'search')
     prs_get.set_defaults(func=get)
 
-    # Create record
-    prs_create = subprs.add_parser(
+
+# Create record
+def prsCreate(obj, conn):
+    prs_create = obj.add_parser(
         'create', help='create record of specific zone')
     setoption(prs_create, 'domain')
-    conn_options(prs_create, server, username, password)
+    conn_options(prs_create, conn)
     prs_create.set_defaults(func=create)
 
-    # Create bulk_records
-    prs_create = subprs.add_parser(
+
+# Create bulk_records
+def prsBulkCreate(obj, conn):
+    prs_create = obj.add_parser(
         'bulk_create', help='create bulk records of specific zone')
     setoption(prs_create, 'infile')
-    conn_options(prs_create, server, username, password)
+    conn_options(prs_create, conn)
+    prs_create.add_argument('--domain', action='store',
+                            help='create records with specify zone')
     prs_create.set_defaults(func=create)
 
-    # Delete record
-    prs_delete = subprs.add_parser(
+
+# Delete record
+def prsDelete(obj, conn):
+    prs_delete = obj.add_parser(
         'delete', help='delete a record of specific zone')
     setoption(prs_delete, 'domain')
-    conn_options(prs_delete, server, username, password)
+    conn_options(prs_delete, conn)
     prs_delete.set_defaults(func=delete)
 
-    # Delete bulk_records
-    prs_delete = subprs.add_parser(
+
+# Delete bulk_records
+def prsBulkDelete(obj, conn):
+    prs_delete = obj.add_parser(
         'bulk_delete', help='delete bulk records of specific zone')
     setoption(prs_delete, 'infile')
-    conn_options(prs_delete, server, username, password)
+    conn_options(prs_delete, conn)
+    prs_delete.add_argument('--domain', action='store',
+                            help='delete records with specify zone')
     prs_delete.set_defaults(func=delete)
 
-    # Retrieve template
-    prs_tmpl_get = subprs.add_parser(
+
+# Retrieve template
+def prsTmplGet(obj, conn):
+    prs_tmpl_get = obj.add_parser(
         'tmpl_get', help='retrieve templates')
     setoption(prs_tmpl_get, 'template')
-    conn_options(prs_tmpl_get, server, username, password)
+    conn_options(prs_tmpl_get, conn)
     prs_tmpl_get.set_defaults(func=template_get)
 
-    # create or update template
-    prs_tmpl_create_update = subprs.add_parser(
-        'tmpl_create_update', help='create or update template')
-    prs_tmpl_create_update.add_argument(
-        '--domain', action='store', required=True,
-        help='create template with specify domain')
-    setoption(prs_tmpl_create_update, 'template', 'update template with ')
-    prs_tmpl_create_update.add_argument(
-        '--dnsaddr', action='store', required=True,
-        help='specify IP address of NS record')
-    prs_tmpl_create_update.add_argument(
-        '--desc', action='store', help='description')
-    conn_options(prs_tmpl_create_update, server, username, password)
-    prs_tmpl_create_update.set_defaults(func=template_create_or_update)
 
-    # delete template
-    prs_tmpl_delete = subprs.add_parser(
+# Delete template
+def prsTmplDelete(obj, conn):
+    prs_tmpl_delete = obj.add_parser(
         'tmpl_delete', help='delete template')
     setoption(prs_tmpl_delete, 'template', required=True)
-    conn_options(prs_tmpl_delete, server, username, password)
+    conn_options(prs_tmpl_delete, conn)
     prs_tmpl_delete.set_defaults(func=template_delete)
 
-    # update SOA serial
-    prs_soa = subprs.add_parser(
-        'soa', help='increase SOA serial')
+
+# Update SOA serial
+def prsSOAUpdate(obj, conn):
+    prs_soa = obj.add_parser('soa', help='update SOA record')
     prs_soa.add_argument('--domain', action='store', required=True,
                             help='specify domain FQDN')
-    conn_options(prs_soa, server, username, password)
+    prs_soa.add_argument('--mname', action='store',
+                         help='specify MNAME of SOA record')
+    prs_soa.add_argument('--rname', action='store',
+                         help='specify RNAME of SOA record')
+    prs_soa.add_argument('--refresh', action='store', type=int,
+                         help='specify REFRESH of SOA record')
+    prs_soa.add_argument('--retry', action='store', type=int,
+                         help='specify RETRY of SOA record')
+    prs_soa.add_argument('--expire', action='store', type=int,
+                         help='specify EXPIRE of SOA record')
+    prs_soa.add_argument('--minimum', action='store', type=int,
+                         help='specify MINIMUM of SOA record')
+    conn_options(prs_soa, conn)
     prs_soa.set_defaults(func=updateSOASerial)
 
-    args = prs.parse_args()
-    return args
+
+# Create zone
+def prsZoneCreate(obj, conn):
+    prs_zone_create = obj.add_parser('zone_create', help='create zone')
+    prs_zone_create.add_argument(
+        '--domain', action='store', required=True, help='specify zone')
+    prs_zone_create.add_argument('--dnsaddr', action='store', required=True,
+        help='specify IP address of DNS master')
+    group_zone_create = prs_zone_create.add_mutually_exclusive_group()
+    group_zone_create.add_argument('-S', action='store_true',
+                                   help='create zone to SLAVE')
+    group_zone_create.add_argument('-N', action='store_true',
+                                   help='create zone to NATIVE')
+    conn_options(prs_zone_create, conn)
+    prs_zone_create.set_defaults(func=createZone)
+
+
+# Delete zone
+def prsZoneDelete(obj, conn):
+    prs_zone_delete = obj.add_parser('zone_delete', help='delete zone')
+    prs_zone_delete.add_argument('--domain', action='store', required=True,
+                                 help='specify zone')
+    conn_options(prs_zone_delete, conn)
+    prs_zone_delete.set_defaults(func=zone_delete)
 
 
 def checkConfig(filename):
@@ -457,7 +572,6 @@ def checkConfig(filename):
 def main():
     import sys
 
-    '''
     try:
         args = parse_options()
         args.func(args)
@@ -467,9 +581,6 @@ def main():
     except UnboundLocalError as e:
         sys.stderr.write("ERROR: %s\n" % e)
         return
-        '''
-    args = parse_options()
-    args.func(args)
 
 if __name__ == "__main__":
     main()
