@@ -69,9 +69,9 @@ class JSONConvert(object):
         from datetime import date
         serial = date.strftime(date.today(), '%Y%m%d') + '01'
         ns = 'ns.' + domain
-        soa = ns + ' hostmaster.' + domain + \
-            ' ' + serial + ' ' + self.refresh + ' ' + self.retry + \
-            ' ' + self.expire + ' ' + str(self.ttl)
+        soa = (ns + ' hostmaster.' + domain +
+            ' ' + serial + ' ' + self.refresh + ' ' + self.retry +
+            ' ' + self.expire + ' ' + str(self.ttl))
         self.record = {
                 'identifier': domain.replace('.', '_'),
                 'description': desc,
@@ -90,20 +90,17 @@ class JSONConvert(object):
             }
         return record_d
 
-    def generateZone(self, domain, template, records):
-        import json
+    def generateZone(self, domain, template, dtype, master=None):
         # If there is a SOA record in records,
         # add self SOA records.
         self.zone = {
             "name": domain,
-            "type": "MASTER",
-            "master": None,
+            "type": dtype,
+            "master": master,
             "templates": [
                 {"identifier": template}
-                ],
-            "records": records.get('records')
+                ]
             }
-        return json.JSONEncoder().encode(self.zone)
 
     def checkkey(self, key, index):
         import re
@@ -145,11 +142,31 @@ class JSONConvert(object):
         import json
         json.load(data, 'utf-8')
 
-    def getSOA(self, record):
+    def getSOA(self, record, content):
         from datetime import date
         new_record = record.copy()
-        mname = record.get('content').split(' ')[0]
-        rname = record.get('content').split(' ')[1]
+
+        if content.get('mname'):
+            mname = content.get('mname')
+        else:
+            mname = record.get('content').split(' ')[0]
+
+        if content.get('rname'):
+            rname = content.get('rname')
+        else:
+            rname = record.get('content').split(' ')[1]
+
+        if content.get('refresh'):
+            self.refresh = str(content.get('refresh'))
+
+        if content.get('retry'):
+            self.retry = str(content.get('retry'))
+
+        if content.get('expire'):
+            self.expire = str(content.get('expire'))
+
+        if content.get('minimum'):
+            self.ttl = str(content.get('minimum'))
 
         today = date.strftime(date.today(), '%Y%m%d')
         cur_serial = record.get('content').split(' ')[2]
@@ -158,7 +175,7 @@ class JSONConvert(object):
         else:
             serial = str(int(cur_serial) + 1)
 
-        new_record['content'] = mname + ' ' + rname + ' ' + \
-            serial + ' ' + self.refresh + ' ' + self.retry + ' ' + \
-            self.expire + ' ' + str(self.ttl)
+        new_record['content'] = (mname + ' ' + rname + ' ' +
+            serial + ' ' + self.refresh + ' ' + self.retry + ' ' +
+            self.expire + ' ' + str(self.ttl))
         return new_record
