@@ -16,12 +16,7 @@
     You should have received a copy of the GNU General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
-from . import connect as conn
-
-
-def unprovide():
-    print("ERROR: This feature does not provide")
-    exit(10)
+import connect as conn
 
 
 # `data' is list.
@@ -29,24 +24,19 @@ def unprovide():
 # 1) {"records": records}
 # 2) {"name": domain, "records": records}
 #This is work around, see also commit 7571109.
-def createZoneRecords(server, token, domain, data, identifier):
+def createZoneRecords(server, token, domain, identifier,
+                      dtype, master=None):
     # ContentType: application/json
     # x-authentication-token: token
+    # dtype: MASTER|SLAVE|NATIVE (default: MASTER)
+    # master: master server ip address when dtype is SLAVE (default: None)
     method = 'PUT'
     uri = 'https://' + server + '/zone'
 
-    for i, v in enumerate(data):
-
-        if i == 0:
-            from converter import JSONConvert
-            obj = JSONConvert(domain)
-            zone = obj.generateZone(domain, identifier, v)
-            conn.tonicDNSClient(uri, method, token, zone)
-
-        else:
-            # method: PUT
-            uri = 'https://' + server + '/zone/' + domain
-            conn.tonicDNSClient(uri, method, token, v)
+    from converter import JSONConvert
+    obj = JSONConvert(domain)
+    obj.generateZone(domain, identifier, dtype, master)
+    conn.tonicDNSClient(uri, method, token, obj.zone)
 
 
 def createRecords(server, token, domain, data):
@@ -81,11 +71,11 @@ def getAllZone(server, token):
     conn.tonicDNSClient(uri, method, token, data=False)
 
 
-def deleteDomain():
+def deleteZone(server, token, domain):
     # x-authentication-token: token
-    # method: DELETE
-    # uri: /zone/:domain
-    unprovide()
+    method = 'DELETE'
+    uri = 'https://' + server + '/zone/' + domain
+    conn.tonicDNSClient(uri, method, token, data=False)
 
 
 def createTemplate(server, token, identifier, template):
@@ -125,18 +115,19 @@ def getAllTemplates(server, token):
     conn.tonicDNSClient(uri, method, token, data=False)
 
 
-def updateSerial(server, token, domain):
+def updateSerial(server, token, soa_content):
     # x-authentication-token: token
 
     # Get SOA record
     # `cur_soa` is current SOA record.
     # `new_soa` is incremental serial SOA record.
     method = 'GET'
-    uri = 'https://' + server + '/zone/' + domain
-    cur_soa, new_soa = conn.tonicDNSClient(uri, method, token, data=False,
-                                           keyword='serial', domain=domain)
+    uri = 'https://' + server + '/zone/' + soa_content.get('domain')
+    cur_soa, new_soa = conn.tonicDNSClient(
+        uri, method, token, data=False, keyword='serial', content=soa_content)
     # set JSON
     from converter import JSONConvert
+    domain = soa_content.get('domain')
     cur_o = JSONConvert(domain)
     new_o = JSONConvert(domain)
     cur_o.records = [cur_soa]
